#include "RTClib.h"
#include <TimeLib.h>
#include <Wire.h>

#define PIN_LAT    26 
#define PIN_SCLK   27 
#define PIN_DATA   33 
#define PIN_BLANK  25 

#define I2C_SCL 18
#define I2C_SDA 19

uint16_t tubes[6][10] = {
 // A   B   C   D   E   F   G   H   I  DP 
  { 1, 10,  6,  4,  3,  2,  8,  9,  5,  7 }, // tube 0
  {11, 20, 16, 14, 13, 12, 18, 19, 15, 17 }, // tube 1
  {21, 30, 26, 24, 23, 22, 28, 29, 25, 27 }, // tube 2
  {32, 41, 37, 35, 34, 33, 39, 40, 36, 38 }, // tube 3
  {42, 52, 48, 45, 44, 43, 50, 51, 46, 49 }, // tube 4
  {54, 63, 59, 57, 56, 55, 61, 62, 58, 60 }  // tube 5
};
  

 uint16_t NineSegmentASCII[26] = {
    //ABCDEFGHIx
    0b1111110000,  // 48  0
    0b0110000100,  // 49  1  
    0b1101000010,  // 50  2  
    0b1000001110,  // 51  3
    0b0110011000,  // 52  4
    0b1011011000,  // 53  5
    0b0011101100,  // 54  6
    0b1000100100,  // 55  7
    0b1111111000,  // 56  8
    0b1100011010,  // 57  9
  };

RTC_DS3231 rtc;

boolean srBuffer[64] = {0};

volatile int tubeBrightness = 9; // 0 to 9
volatile int updating_register = 0;

int levelOn[] =  { 2, 3, 4, 5, 2, 5, 8, 9, 10, 1};
int levelOff[] = { 3, 3, 3, 3, 1, 2, 3, 3, 2, 0};

int onDuration = levelOn[tubeBrightness];
int offDuration = levelOff[tubeBrightness];

volatile int interruptState = 0;
volatile int interruptCounter = 0;

unsigned long previousMillis = 0;

hw_timer_t * timer = NULL;
portMUX_TYPE timerMux = portMUX_INITIALIZER_UNLOCKED;

/////////////////////////////////////////////////
/////////////////////////////////////////////////

void IRAM_ATTR onTimer() {

    portENTER_CRITICAL_ISR(&timerMux);
    if (updating_register == 0){
        if (offDuration == 0){
          digitalWrite(PIN_BLANK, LOW);
        } else {
    
          interruptCounter++;
    
          if (interruptState == 0){
            if (interruptCounter >= offDuration){  
            interruptState = !interruptState;
            interruptCounter = 0;
            digitalWrite(PIN_BLANK, LOW);
            }
          } else {
            if (interruptCounter >= onDuration){
            interruptState = !interruptState;
            interruptCounter = 0;
            digitalWrite(PIN_BLANK, HIGH);
            }
          }
    
        }
    }
    portEXIT_CRITICAL_ISR(&timerMux);

}

/////////////////////////////////////////////////
/////////////////////////////////////////////////

void  loadCharacter(char character, int tube){

    uint16_t letter = NineSegmentASCII[(int)character - 48];

    for (int j = 0; j < 10; j++){
        bool x = (letter >> (9-j)) & 1;
        int pos = tubes[tube][j];
        if (x == 1){
            srBuffer[pos] = HIGH;
        } else {
            // srBuffer[pos] = LOW;
        }
    }
  
}

/////////////////////////////////////////////////
/////////////////////////////////////////////////

void clockIn(){

    updating_register = 1;
    delayMicroseconds(1);
    for (int i = 0; i < 64; i++) {
        digitalWrite(PIN_DATA, srBuffer[63 - i]);
        digitalWrite(PIN_SCLK, LOW);
        digitalWrite(PIN_SCLK, HIGH);
    }
    digitalWrite(PIN_BLANK, HIGH);
    digitalWrite(PIN_LAT, HIGH);
    digitalWrite(PIN_LAT, LOW);
    digitalWrite(PIN_BLANK, LOW);
    delayMicroseconds(1);
    updating_register = 0;

}

/////////////////////////////////////////////////
/////////////////////////////////////////////////

void setup() {

    pinMode(PIN_SCLK, OUTPUT);
    pinMode(PIN_DATA, OUTPUT);
    pinMode(PIN_LAT, OUTPUT);
    pinMode(PIN_BLANK, OUTPUT);
  
    digitalWrite(PIN_SCLK, LOW);
    digitalWrite(PIN_DATA, LOW);
    digitalWrite(PIN_BLANK, LOW);
    digitalWrite(PIN_LAT, LOW);

    Serial.begin(115200); while (!Serial); delay(200);

    Wire.begin (I2C_SDA, I2C_SCL); 

    delay(500); // some sort of delay is necessary

    rtc.begin();

    //if (1) {
    if (rtc.lostPower()) {
    
        rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
        DateTime now = rtc.now();
        int _year = now.year();
        int _month = now.month();
        int _day = now.day();
        int _hour = now.hour();
        int _minute = now.minute();
        int _second = now.second();
         rtc.adjust(DateTime(_year, _month, _day, _hour, _minute, _second));
        //rtc.adjust(DateTime(2000, 1, 1, 21, 59, 45));
      
    }

    timer = timerBegin(0, 80, true);
    timerAttachInterrupt(timer, &onTimer, true);
    timerAlarmWrite(timer, 1000, true); // 250ms
    timerAlarmEnable(timer);

}

/////////////////////////////////////////////////
/////////////////////////////////////////////////

void loop() {

    unsigned long currentMillis = millis();

    if ((currentMillis - previousMillis) >= 250) {

        DateTime now = rtc.now();
      
        int hours = now.hour();
        int minutes = now.minute();
        int seconds = now.second();
    
        memset(srBuffer, 0, sizeof(srBuffer));

        char disp[7] = "      ";
                
        sprintf(disp, "%02d%02d%02d", hours, minutes, seconds);
    
        loadCharacter(disp[0], 0);
        loadCharacter(disp[1], 1);
        loadCharacter(disp[2], 2);
        loadCharacter(disp[3], 3);
        loadCharacter(disp[4], 4);
        loadCharacter(disp[5], 5);
    
        srBuffer[17] = 1;
        srBuffer[38] = 1;

        clockIn();
        delayMicroseconds(1);
        clockIn();
    
        previousMillis = currentMillis;

    }

}
